/* ezcdb.h
** ezcdb application header
** wcm, 2010.05.28 - 2010.11.09
** ===
*/

#ifndef EZCDB_H
#define EZCDB_H 1

/* libasagna: */
#include "die.h"
#include "ioq.h"
#include "ioq_std.h"
#include "sysstr.h"

#define EZCDB_VERSION "0.01"


/* defined non-NULL for all ezcdb_*.c: */
extern const char *progname;
extern const char *usage_prog;

/* defined non-NULL only by ezcdb_master.c */
extern const char *progmode;
extern const char *usage_mode;


/* using ioq_std for stderr output: */
#define eputs(...) \
  {ioq_vputs(ioq2, __VA_ARGS__, "\n"); ioq_flush(ioq2); }

#define usage() \
  {\
      if(progmode != NULL){\
          eputs("usage: ", progname, " ", progmode, " ", usage_mode);\
       }else{\
          eputs("usage: ", progname, " ", usage_prog);\
       }\
  }
  

#define version() \
  eputs(progname, ": version: ", EZCDB_VERSION);

#define die_usage() \
  {usage(); die(100);}

#define warn(...) \
  {\
      if(progmode != NULL){\
          eputs(progname, " ", progmode, ": warning: ", __VA_ARGS__);\
      }else{\
          eputs(progname, ": warning: ", __VA_ARGS__);\
      }\
  }

#define fatal(e, ...) \
  {\
      if(progmode != NULL){\
          eputs(progname, " ", progmode, ": fatal: ", __VA_ARGS__);\
      }else{\
          eputs(progname, ": fatal: ", __VA_ARGS__);\
      }\
      die(e);\
  }

#define fatal_syserr(...) \
  fatal(111, __VA_ARGS__, ": ", \
        sysstr_errno_mesg(errno), " (", \
        sysstr_errno(errno), ")")

#define fatal_usage(...) \
  {\
      if(progmode != NULL){\
          eputs(progname, " ", progmode, ": usage error: ", __VA_ARGS__);\
      }else{\
          eputs(progname, ": usage error: ", __VA_ARGS__);\
      }\
      die_usage();\
  }


#if 0
#define debug(...) \
  eputs("debug: ", __VA_ARGS__)
#endif
#define debug(...)

/* ezcdb modules ("modes"): */
extern int ezcdb_cross(int argc, char *argv[]);
extern int ezcdb_dump(int argc, char *argv[]);
extern int ezcdb_dupes(int argc, char *argv[]);
extern int ezcdb_get(int argc, char *argv[]);
extern int ezcdb_grep(int argc, char *argv[]);
extern int ezcdb_keys(int argc, char *argv[]);
extern int ezcdb_make(int argc, char *argv[]);
extern int ezcdb_match(int argc, char *argv[]);
extern int ezcdb_merge(int argc, char *argv[]);
extern int ezcdb_purge(int argc, char *argv[]);
extern int ezcdb_stats(int argc, char *argv[]);


#endif /* EZCDB_H */
/* eof (ezcdb.h) */
